import six


class ImageDetail(object):
    """
        A representation of server image details. This is typically the build image or template used to deploy a server. It contains information about the OS, architecture etc.

        Args:
            id (str): id, e.g. "ami-abcdef"
            name (str): name, e.g. "Ubuntu Linux 16.04 LTS 64 bit"
            type (str): ``Windows`` or ``Linux``
            distribution (str): distribution or subtype, e.g. ``Server 2012`` or ``CentOS``
            version (str): version e.g. ``V2`` or ``16.04.1``
            architecture (str): architecture e.g ``i386``, ``x86_64``

    """  # noqa

    def __init__(self, id=None, name=None, type=None, distribution=None,
                 version=None, architecture=None):
        self._id = id
        self._name = name
        self._type = type
        self._distribution = distribution
        self._version = version
        self._architecture = architecture

        self._check_not_none_str_value('id', self._id)

    def _check_not_none_str_value(self, name, value):
        if value is None or not isinstance(value, six.string_types):
            raise ValueError('%s must be a string and have a value' % name)

    def serialize(self):
        """Serialize the contents"""

        return {
            "id": self._id,
            "name": self._name,
            "architecture": self._architecture,
            "distribution": self._distribution,
            "type": self._type,
            "version": self._version,
        }


class ImageDetailMap(object):
    """
        A utility class that is able to look up common server images used for several servers using a dict mapping ids to the details.

        Args:
            mapping (dict): A dict with str keys and tuple values

        Example:
            This shows initializing a mapping and looking up images::

                m = ImageDetailMap([
                    'ubuntu16': ('Ubuntu Linux 16.04 LTS 64 bit', 'Linux', 'Ubuntu', '16.04', 'x64'),
                    'w2012R2': ('Windows Server 2012R2 Standard', 'Windows', 'Server 2012', 'R2', 'x64'),
                ])

                m.get('no-match')                       # Returns an Image with None
                m.get('ubuntu16')                       # Matches the image with "ubuntu16"
                m.get('ubuntu16', architecture='i386')  # Matches the image with "ubuntu16" and overrides the architecture
    """  # noqa

    def __init__(self, mapping=None):
        self._mapping = mapping

    def get(self, id=None, name=None, version=None, type=None,
            architecture=None, distribution=None):
        """
            Lookup an image. If none is found, an image is returned with no data in it.
            Use the arguments to override individual details.

            Args:
                name (str): optional name
                type (str): optional type
                distribution (str): optional distribution
                version (str): optional version
                architecture (str): optional architecture

            Returns: :py:class:`nflex_connector_utils.image_detail.ImageDetail`
        """  # noqa

        mapped_image = self._mapping.get(id)
        if id is None:
            raise ValueError("id must be a string and have a value")
        if mapped_image is None:
            (mname, mtype, mdist, mversion, march) = tuple([None] * 5)
        else:
            (mname, mtype, mdist, mversion, march) = mapped_image

        mname = mname or name
        mtype = mtype or type
        mversion = mversion or version
        march = march or architecture
        mdist = mdist or distribution

        return ImageDetail(
            id=id,
            name=mname,
            type=mtype,
            distribution=mdist,
            version=mversion,
            architecture=march,
        )



from datetime import datetime
import six
from . import Connections
from . import Metadata
from . import convert_datetime


class Resource(object):
    """
        A representation of a resource. This contains all data common to all resources.

        Args:
            id (str): Unique identifier of this resource type. The ``(id, type)`` tuple uniquely identifies a resource.
            type (str): Type of a resource, e.g. ``server``, ``network``, ``volume``, ...
            name (str): Human readable name of the resource
            provider_created_at (str or datetime): An optional string or datetime object when the resource was created. This should never change.
            native_portal_link (str): An optional url to a page on a provider portal with details of the resource.
            region (:py:class:`nflex_connector_utils.locations.Region`): An optional :py:class:`nflex_connector_utils.locations.Region` object that associates the resource with a CMP location.
            locations (:py:class:`nflex_connector_utils.locations.Locations`): An optional :py:class:`nflex_connector_utils.locations.Locations` objects that provides extended location information.
            connections (:py:class:`nflex_connector_utils.connections.Connections`)_: An optional :py:class:`nflex_connector_utils.connections.Connections` object
            metadata (:py:class:`nflex_connector_utils.metadata.Metadata`): An optional :py:class:`nflex_connector_utils.metadata.Metadata` object

    """  # noqa
    def __init__(self, id=None, name=None, type=None, region=None,
                 locations=None, provider_created_at=None, metadata=None,
                 native_portal_link=None, connections=None):
        self.id = id
        self.name = name
        self.type = type
        self._check_not_none_str_value('id', self.id)
        self._check_not_none_str_value('name', self.name)
        self._check_not_none_str_value('type', self.type)

        self._provider_created_at = provider_created_at

        if connections is None:
            connections = Connections()
        self._connections = connections

        if metadata is None:
            metadata = Metadata()
        self._metadata = metadata
        self._region = region
        self._locations = locations
        self._native_portal_link = native_portal_link

    def _check_not_none_str_value(self, name, value):
        if value is None or not isinstance(value, six.string_types):
            raise ValueError('%s must be a string and have a value' % name)

    def serialize(self):
        """Serialize the contents"""

        provider_created_at = convert_datetime(self._provider_created_at)

        regions = []
        if self._region is not None:
            regions = [self._region.serialize()]

        results = {
            "id": self.id,
            "type": self.type,
            "base": {
                "name": self.name,
                "provider_created_at": provider_created_at,
                'last_seen_at': convert_datetime(datetime.utcnow()),
            },
            "connections": self._connections.serialize(),
            "metadata": self._metadata.serialize(),
        }

        if self._native_portal_link is not None:
            results['base']['native_portal_link'] = self._native_portal_link

        locations = []
        if self._locations is not None:
            locations = self._locations.serialize()
        if len(locations) > 0:
            results['base']['locations'] = locations
        else:
            regions = []
            if self._region is not None:
                regions = [self._region.serialize()]
            results['base']['regions'] = regions

        return results



from . import Resource


class Volume(Resource):
    """
        A representation of a volume. See

        Args:
            base (base): See :py:class:`nflex_connector_utils.resource.Resource` for common resource args.
            size_b (int): Optional size in bytes
            encrypted (bool): Optional, set to true if the volume is encrypted
            iops (int): Optional iops
            zone_name (str): Optional zone name
    """  # noqa

    def __init__(self, size_b=None, encrypted=None, iops=None, zone_name=None,
                 **kwargs):
        super(Volume, self).__init__(type='volume', **kwargs)
        self._size_b = size_b
        self._encrypted = encrypted
        self._iops = iops
        self._zone_name = zone_name

    @property
    def size_b(self):
        '''Size in bytes'''
        return self._size_b

    def serialize(self):
        """Serialize the contents"""

        data = super(Volume, self).serialize()

        iops = None
        if self._iops is not None:
            iops = int(self._iops)

        data['details'] = {
            self.type: {
                "encrypted": self._encrypted,
                "iops": iops,
                "size_b": self._size_b,
            }
        }

        if self._zone_name is not None:
            data['details']['volume']['zone_name'] = self._zone_name

        return data



from . import Resource


class SaasUser(Resource):
    """
        A representation of an Saas User

        Args:
            base (base): See :py:class:`nflex_connector_utils.resource.Resource` for common resource args.
            user_id (str): Id of the Saas User
            avatar_url (str): Avatal URL of the user
            phone (str): Phone number of the user
            address (str): Address of the Saas User
            language (str): Preferred language of the user
            is_active (Boolean): To check if the user is active or not
            disk_quota_b (str): The Storage allocated to the user
            disk_used_b (str): The Storage used by the user


    """  # noqa

    def __init__(self, avatar_url=None, phone=None, address=None,
                 language=None, is_active=None, disk_quota_b=None,
                 disk_used_b=None, email=None, country=None, **kwargs):

        super(SaasUser, self).__init__(type='saas_user', **kwargs)

        self._avatar_url = avatar_url
        self._phone = phone
        self._address = address
        self._language = language
        self._is_active = is_active
        self._disk_quota_b = disk_quota_b
        self._disk_used_b = disk_used_b
        self._email = email
        self._country = country

    def serialize(self):
        """Serialize the contents"""

        data = super(SaasUser, self).serialize()

        data['details'] = {
            self.type: {
                "avatar_url": self._avatar_url,
                "phone": self._phone,
                "address": self._address,
                "is_active": self._is_active,
                "disk_quota_b": self._disk_quota_b,
                "disk_used_b": self._disk_used_b,
                "email": self._email,
                "country": self._country,
                "language": self._language,
            }
        }

        return data



from pip.req import parse_requirements
import uuid
import versioneer
from setuptools import setup, find_packages

requirements = parse_requirements('requirements.txt', session=uuid.uuid1())
install_requires = [str(r.req) for r in requirements]
description = """nflex-connector-utils provides a suite of tools to produce data structures when implementing resources connector nflex modules."""  # noqa

long_description = """
nflex-connector-utils provides a suite of tools to produce data structures
when implementing resources connector nflex modules.
See readthedocs_ for the documentation and github_ for the source.

.. _github: https://github.com/ntt-nflex/nflex_connector_utils
.. _readthedocs: http://nflex-connector-utils.readthedocs.io/en/latest/index.html
"""  # noqa

setup(
    name="nflex-connector-utils",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description=description,
    long_description=long_description,
    url='http://www.ntt.com',
    author='NTT communications',
    licence='GNU General Public License v2 (GPLv2)',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development',
    ],
)



from . import Resource


class ComputePool(Resource):
    """
        A representation of a compute pool.

        Args:
            base (base): See :py:class:`nflex_connector_utils.resource.Resource` for common resource args.
            cpu_hz (int): cpu in Hz (optional)
            memory_b (int): memory in bytes (optional)
            storage_b (int): storage in bytes (optional)
            billing_tag (str): billing tag for the compute pool (optional)
    """  # noqa

    def __init__(self, cpu_hz=None, memory_b=None, storage_b=None,
                 billing_tag=None, **kwargs):
        super(ComputePool, self).__init__(type='compute_pool', **kwargs)
        self._cpu_hz = cpu_hz
        self._memory_b = memory_b
        self._storage_b = storage_b
        self._billing_tag = billing_tag

    def serialize(self):
        """Serialize the contents"""

        data = super(ComputePool, self).serialize()
        details = {
            self.type: {
                "cpu_hz": self._cpu_hz,
                "memory_b": self._memory_b,
                "storage_b": self._storage_b,
            }
        }

        if self._billing_tag is not None:
            details[self.type]['billing_tag'] = self._billing_tag

        data['details'] = details
        return data
